@@ -1,6 +1,6 @@
 package App::Mobirc;
 use Moose;
-with 'App::Mobirc::Role::Pluggable', 'App::Mobirc::Role::Context';
+with 'App::Mobirc::Role::Context', 'MooseX::Plaggerize', 'MooseX::Plaggerize::PluginLoader';
 use 5.00800;
 use Scalar::Util qw/blessed/;
 use POE;
@@ -12,7 +12,7 @@ use Carp;
 use App::Mobirc::Model::Server;
 use Encode;
 
-our $VERSION = '0.99_01';
+our $VERSION = '0.99_02';
 
 has server => (
     is      => 'ro',
@@ -32,20 +32,27 @@ around 'new' => sub {
     my $config = App::Mobirc::ConfigLoader->load($config_stuff); # TODO: use coercing
 
     my $self = $next->( $class, config => $config );
-    $self->load_plugins;
+
+    $self->_load_plugins();
 
     return $self;
 };
 
+sub _load_plugins {
+    my $self = shift;
+    for my $plugin (@{ $self->config->{plugin} }) {
+        $plugin->{module} =~ s/^App::Mobirc::Plugin:://;
+        $self->load_plugin( $plugin );
+    }
+}
+
 sub run {
     my $self = shift;
     die "this is instance method" unless blessed $self;
 
-    for my $code (@{$self->get_hook_codes('run_component')}) {
-        $code->($self);
-    }
+    $self->run_hook('run_component');
 
-    App::Mobirc::HTTPD->init($self->config);
+    App::Mobirc::HTTPD->init($self->config, $self);
 
     $poe_kernel->run();
 }
@@ -89,26 +96,6 @@ run server
 
 register hook
 
-=item get_hook_codes
-
-get hook codes
-
-=item add_channel
-
-register channel object
-
-=item channels
-
-get a channel objects.
-
-=item get_channel
-
-get a channel
-
-=item delete_channel
-
-delete channel
-
 =back
 
 =head1 TODO
