@@ -1,6 +1,6 @@
 package App::Mobirc::Plugin::Component::IRCClient;
 use strict;
-use warnings;
+use MooseX::Plaggerize::Plugin;
 
 use POE;
 use POE::Sugar::Args;
@@ -12,27 +12,70 @@ use Carp;
 use App::Mobirc::Model::Message;
 use App::Mobirc::Util;
 
-sub register {
-    my ($class, $global_context, $conf) = @_;
+has ping_delay => (
+    is      => 'ro',
+    isa     => 'Int',
+    default => 30,
+);
 
-    DEBUG "register ircclient component";
-    $conf->{ping_delay}       ||= 30;
-    $conf->{reconnect_delay}  ||= 10;
+has reconnect_delay => (
+    is      => 'ro',
+    isa     => 'Int',
+    default => 10,
+);
 
-    $global_context->register_hook(
-        'run_component' => sub { _init($conf, shift) },
-    );
-    $global_context->register_hook(
-        'process_command' => sub { my ($global_context, $command, $channel) = @_;  _process_command($conf, $global_context, $command, $channel) },
-    );
-}
+has incode => (
+    is      => 'ro',
+    isa     => 'Str',
+    default => 'UTF-8',
+);
 
-sub _process_command {
-    my ($conf, $global_context, $command, $channel) = @_;
+has nick => (
+    is       => 'ro',
+    isa      => 'Str',
+    required => 1,
+);
 
-    my $irc_incode = $conf->{incode};
-    if ($command && $channel->name =~ /^[#*%]/) {
-        if ($command =~ m{^/}) {
+has username => (
+    is      => 'ro',
+    isa     => 'Str',
+    default => 'mobirc user',
+);
+
+has groups => (
+    is  => 'ro',
+    isa => 'HashRef',
+);
+
+has desc => (
+    is      => 'ro',
+    isa     => 'Str',
+    default => '',
+);
+
+has server => (
+    is       => 'ro',
+    isa      => 'Str',
+    required => 1,
+);
+
+has port => (
+    is       => 'ro',
+    isa      => 'Int',
+    required => 1,
+);
+
+has password => (
+    is  => 'ro',
+    isa => 'Str',
+);
+
+hook process_command => sub {
+    my ( $self, $global_context, $command, $channel ) = @_;
+
+    my $irc_incode = $self->incode;
+    if ( $command && $channel->name =~ /^[#*%]/ ) {
+        if ( $command =~ m{^/} ) {
             DEBUG "SENDING COMMAND";
             $command =~ s!^/!!g;
 
@@ -40,22 +83,25 @@ sub _process_command {
               map { encode( $irc_incode, $_ ) } split /\s+/,
               $command;
 
-            $poe_kernel->post('mobirc_irc', @args);
-        } else {
+            $poe_kernel->post( 'mobirc_irc', @args );
+        }
+        else {
             DEBUG "NORMAL PRIVMSG";
 
             $poe_kernel->post( 'mobirc_irc',
                 privmsg => encode( $irc_incode, $channel->name ) =>
-                encode( $irc_incode, $command ) );
+                  encode( $irc_incode, $command ) );
 
             DEBUG "Sending command $command";
+
             # FIXME: httpd $B4X78$J$$7o(B
-            if ($global_context->config->{httpd}->{echo} eq true) {
+            if ( $global_context->config->{httpd}->{echo} eq true ) {
                 $channel->add_message(
                     App::Mobirc::Model::Message->new(
                         who => decode(
                             $irc_incode,
-                            $poe_kernel->alias_resolve('irc_session')->get_heap->{irc}->nick_name
+                            $poe_kernel->alias_resolve('irc_session')
+                              ->get_heap->{irc}->nick_name
                         ),
                         body  => $command,
                         class => 'public',
@@ -66,48 +112,49 @@ sub _process_command {
         return true;
     }
     return false;
-}
+};
 
-sub _init {
-    my ($config, $global_context) = @_;
+hook 'run_component' => sub {
+    my ( $self, $global_context ) = @_;
 
     DEBUG "initialize ircclient";
+
     # irc component
     my $irc = POE::Component::IRC->spawn(
         Alias    => 'mobirc_irc',
-        Nick     => $config->{nick},
-        Username => $config->{username},
-        Ircname  => $config->{desc},
-        Server   => $config->{server},
-        Port     => $config->{port},
-        Password => $config->{password}
+        Nick     => $self->nick,
+        Username => $self->username,
+        Ircname  => $self->desc,
+        Server   => $self->server,
+        Port     => $self->port,
+        Password => $self->password,
     );
 
     POE::Session->create(
         heap => {
             seen_traffic   => false,
             disconnect_msg => true,
-            config         => $config,
+            config         => $self,
             irc            => $irc,
             global_context => $global_context,
         },
         inline_states => {
-            _start           => \&on_irc_start,
-            _default         => \&on_irc_default,
+            _start   => \&on_irc_start,
+            _default => \&on_irc_default,
 
-            irc_001          => \&on_irc_001,
-            irc_join         => \&on_irc_join,
-            irc_part         => \&on_irc_part,
-            irc_public       => \&on_irc_public,
-            irc_notice       => \&on_irc_notice,
-            irc_topic        => \&on_irc_topic,
-            irc_332          => \&on_irc_topicraw,
-            irc_ctcp_action  => \&on_irc_ctcp_action,
-            irc_kick         => \&on_irc_kick,
-            irc_snotice      => \&on_irc_snotice,
+            irc_001         => \&on_irc_001,
+            irc_join        => \&on_irc_join,
+            irc_part        => \&on_irc_part,
+            irc_public      => \&on_irc_public,
+            irc_notice      => \&on_irc_notice,
+            irc_topic       => \&on_irc_topic,
+            irc_332         => \&on_irc_topicraw,
+            irc_ctcp_action => \&on_irc_ctcp_action,
+            irc_kick        => \&on_irc_kick,
+            irc_snotice     => \&on_irc_snotice,
 
-            autoping         => \&do_autoping,
-            connect          => \&do_connect,
+            autoping => \&do_autoping,
+            connect  => \&do_connect,
 
             irc_disconnected => \&on_irc_reconnect,
             irc_error        => \&on_irc_reconnect,
@@ -116,9 +163,8 @@ sub _init {
     );
 
     $global_context->add_channel(
-        App::Mobirc::Model::Channel->new($global_context, U('*server*'))
-    );
-}
+        App::Mobirc::Model::Channel->new( $global_context, U('*server*') ) );
+};
 
 # -------------------------------------------------------------------------
 
@@ -143,11 +189,12 @@ sub on_irc_001 {
 
     DEBUG "CONNECTED";
 
-    my $channel = $poe->heap->{global_context}->get_channel(decode( 'utf8', '*server*' ));
+    my $channel =
+      $poe->heap->{global_context}->get_channel( decode( 'utf8', '*server*' ) );
     $channel->add_message(
         App::Mobirc::Model::Message->new(
             who   => undef,
-            body  => decode('utf8', 'Connected to irc server!'),
+            body  => decode( 'utf8', 'Connected to irc server!' ),
             class => 'connect',
         )
     );
@@ -161,7 +208,7 @@ sub on_irc_join {
 
     DEBUG "JOIN";
 
-    my ($who, $channel_name) = _get_args($poe);
+    my ( $who, $channel_name ) = _get_args($poe);
 
     $who =~ s/!.*//;
 
@@ -172,11 +219,10 @@ sub on_irc_join {
     # create channel
     my $channel = $poe->heap->{global_context}->get_channel($channel_name);
     unless ($channel) {
-        $channel = App::Mobirc::Model::Channel->new(
-            $poe->heap->{global_context},
-            $channel_name,
-        );
-        $poe->heap->{global_context}->add_channel( $channel );
+        $channel =
+          App::Mobirc::Model::Channel->new( $poe->heap->{global_context},
+            $channel_name, );
+        $poe->heap->{global_context}->add_channel($channel);
     }
 
     my $irc = $poe->heap->{irc};
@@ -196,7 +242,7 @@ sub on_irc_join {
 sub on_irc_part {
     my $poe = sweet_args;
 
-    my ($who, $channel_name, $msg) = _get_args($poe);
+    my ( $who, $channel_name, $msg ) = _get_args($poe);
 
     $who =~ s/!.*//;
 
@@ -232,7 +278,7 @@ sub on_irc_public {
 
     DEBUG "IRC PUBLIC";
 
-    my ($who, $channel_name, $msg) = _get_args($poe);
+    my ( $who, $channel_name, $msg ) = _get_args($poe);
 
     $who =~ s/!.*//;
 
@@ -248,6 +294,7 @@ sub on_irc_public {
         )
     );
     my $irc = $poe->heap->{irc};
+
     if ( $who eq $irc->nick_name ) {
         DEBUG "CLEAR UNREAD";
         $channel->clear_unread;
@@ -260,13 +307,14 @@ sub on_irc_public {
 sub on_irc_notice {
     my $poe = sweet_args;
 
-    my ($who, $channel_name, $msg) = _get_args($poe);
+    my ( $who, $channel_name, $msg ) = _get_args($poe);
 
     DEBUG "IRC NOTICE $who $channel_name $msg";
 
-    for my $code (@{ $poe->heap->{global_context}->get_hook_codes('on_irc_notice') }) {
-        my $finished = $code->($poe, $who, $channel_name, $msg);
-        return if $finished;
+    if ( $poe->heap->{global_context}
+        ->run_hook_first( 'on_irc_notice', $poe, $who, $channel_name, $msg ) )
+    {
+        return;
     }
 
     $who =~ s/!.*//;
@@ -288,7 +336,7 @@ sub on_irc_notice {
 sub on_irc_topic {
     my $poe = sweet_args;
 
-    my ($who, $channel_name, $topic) = _get_args($poe);
+    my ( $who, $channel_name, $topic ) = _get_args($poe);
 
     $who =~ s/!.*//;
 
@@ -305,34 +353,35 @@ sub on_irc_topic {
         )
     );
 
-    $poe->heap->{seen_traffic}                  = true;
-    $poe->heap->{disconnect_msg}                = true;
+    $poe->heap->{seen_traffic}   = true;
+    $poe->heap->{disconnect_msg} = true;
 }
 
 sub on_irc_topicraw {
     my $poe = sweet_args;
 
-    my ($x, $y, $dat) = _get_args($poe);
+    my ( $x, $y, $dat ) = _get_args($poe);
 
     my ( $channel, $topic ) = @{$dat};
 
     DEBUG "SET TOPIC RAW: $channel => $topic";
 
-    $poe->heap->{global_context}->get_channel(normalize_channel_name($channel))->topic($topic);
-    $poe->heap->{seen_traffic}                  = true;
-    $poe->heap->{disconnect_msg}                = true;
+    $poe->heap->{global_context}
+      ->get_channel( normalize_channel_name($channel) )->topic($topic);
+    $poe->heap->{seen_traffic}   = true;
+    $poe->heap->{disconnect_msg} = true;
 }
 
 sub on_irc_ctcp_action {
     my $poe = sweet_args;
 
-    my ($who, $channel_name, $msg) = _get_args($poe);
+    my ( $who, $channel_name, $msg ) = _get_args($poe);
 
     $who =~ s/!.*//;
     $channel_name = $channel_name->[0];
 
     my $channel = $poe->heap->{global_context}->get_channel($channel_name);
-    my $body = sprintf(decode('utf8', "* %s %s"), $who, $msg);
+    my $body = sprintf( decode( 'utf8', "* %s %s" ), $who, $msg );
     $channel->add_message(
         App::Mobirc::Model::Message->new(
             who   => undef,
@@ -350,7 +399,7 @@ sub on_irc_kick {
 
     DEBUG "DNBKICK";
 
-    my ($kicker, $channel_name, $kickee, $msg) = _get_args($poe);
+    my ( $kicker, $channel_name, $kickee, $msg ) = _get_args($poe);
     $msg ||= 'Flooder';
 
     $kicker =~ s/!.*//;
@@ -376,7 +425,8 @@ sub do_connect {
 sub do_autoping {
     my $poe = sweet_args;
 
-    $poe->kernel->post( mobirc_irc => time ) unless $poe->heap->{seen_traffic};
+    $poe->kernel->post( mobirc_irc => time )
+      unless $poe->heap->{seen_traffic};
     $poe->heap->{seen_traffic} = false;
     $poe->kernel->delay( autoping => $poe->heap->{config}->{ping_delay} );
 }
@@ -384,11 +434,11 @@ sub do_autoping {
 sub on_irc_snotice {
     my $poe = sweet_args;
 
-    my ($message, ) = _get_args($poe);
+    my ( $message, ) = _get_args($poe);
 
     DEBUG "getting snotice : $message";
 
-    my $channel = $poe->heap->{global_context}->get_channel(U('*server*' ));
+    my $channel = $poe->heap->{global_context}->get_channel( U('*server*') );
     $channel->add_message(
         App::Mobirc::Model::Message->new(
             who   => undef,
@@ -403,11 +453,16 @@ sub on_irc_reconnect {
 
     DEBUG "!RECONNECT! " . $poe->heap->{disconnect_msg};
     if ( $poe->heap->{disconnect_msg} ) {
-        my $channel = $poe->heap->{global_context}->get_channel(decode( 'utf8', '*server*' ));
+        my $channel =
+          $poe->heap->{global_context}
+          ->get_channel( decode( 'utf8', '*server*' ) );
         $channel->add_message(
             App::Mobirc::Model::Message->new(
-                who   => undef,
-                body  => decode( 'utf8', 'Disconnected from irc server, trying to reconnect...'),
+                who  => undef,
+                body => decode(
+                    'utf8',
+                    'Disconnected from irc server, trying to reconnect...'
+                ),
                 class => 'reconnect',
             )
         );
@@ -421,11 +476,13 @@ sub _get_args {
     my $poe = shift;
 
     my @ret;
-    for my $elem (@{$poe->args}) {
-        if ( ref $elem && ref $elem eq 'ARRAY') {
-            push @ret, [map { decode($poe->heap->{config}->{incode}, $_) } @$elem];
-        } else {
-            push @ret, decode($poe->heap->{config}->{incode}, $elem);
+    for my $elem ( @{ $poe->args } ) {
+        if ( ref $elem && ref $elem eq 'ARRAY' ) {
+            push @ret,
+              [ map { decode( $poe->heap->{config}->{incode}, $_ ) } @$elem ];
+        }
+        else {
+            push @ret, decode( $poe->heap->{config}->{incode}, $elem );
         }
     }
     return @ret;
